@@ -647,34 +647,7 @@ secr.fit <- function (capthist,  model = list(D~1, g0~1, sigma~1), mask = NULL,
   
   data <- prepareSessionData(capthist, mask, details$maskusage, design, design0, detectfn, 
                              groups, fixed, hcov, details)
-  ############################################
-  # Single evaluation option
-  ############################################
-  .localstuff$iter <- 0
-  if (details$LLonly) {
-    if (is.null(start) | is.list(start))
-      stop ("must provide vector of beta parameter values in 'start'")
-    LL <- - loglikefn (
-      beta = start,
-      parindx    = parindx,
-      link       = link,
-      fixedpar   = fixed,
-      designD    = designD,
-      designNE   = designNE,
-      design     = design,
-      design0    = design0,
-      detectfn   = detectfn,
-      learnedresponse = learnedresponse,
-      sessionlevels = session(capthist),
-      CL         = CL,
-      data       = data,
-      details = details
-    )
-    
-    return(c(logLik=LL))
-  }
-  ############################################
-  
+
   ############################################
   # Start values (model-specific)
   # 'start' is vector of beta values (i.e. transformed) or a list 
@@ -820,6 +793,38 @@ secr.fit <- function (capthist,  model = list(D~1, g0~1, sigma~1), mask = NULL,
   
   setNumThreads(ncores, details$stackSize)
   
+  ############################################
+  # Single evaluation option
+  ############################################
+  .localstuff$iter <- 0
+  if (details$LLonly) {
+      if (is.null(start) || any(is.na(start)))   ## is.list(start))
+          stop ("must provide full vector of beta parameter values in 'start'")
+      prep <- proc.time()
+      LL <- - loglikefn (
+          beta = start,
+          parindx    = parindx,
+          link       = link,
+          fixedpar   = fixed,
+          designD    = designD,
+          designNE   = designNE,
+          design     = design,
+          design0    = design0,
+          detectfn   = detectfn,
+          learnedresponse = learnedresponse,
+          sessionlevels = session(capthist),
+          CL         = CL,
+          data       = data,
+          details = details
+      )
+      out <- c(logLik=LL)
+      attr(out, "npar") <- length(unlist(parindx))
+      attr(out, "preptime") <- (prep-ptm)[3]
+      attr(out, "LLtime") <- (proc.time() - prep)[3]
+      return(out)
+  }
+  ############################################
+ 
   ############################################
   ## estimate overdispersion by simulation
   ############################################
