@@ -15,8 +15,8 @@
 ## e.g. Roger Peng https://stat.ethz.ch/pipermail/r-devel/2009-March/052883.html
 
 .localstuff <- new.env()
-.localstuff$packageType <- ' pre-release'
-## .localstuff$packageType <- ''
+##.localstuff$packageType <- ' pre-release'
+.localstuff$packageType <- ''
 .localstuff$validdetectors <- c('single','multi','proximity','count', 
     'polygonX', 'transectX', 'signal', 'polygon', 'transect', 
     'capped', 'null','null','null','null', 'telemetry', 'signalnoise')
