@@ -17,7 +17,7 @@ writeGPS <- function (xy, o = "garmin", F = "usb:", proj = '+proj=nzmg')  {
     }
     else {
         ## express as lat long
-        if (require (rgdal)) {
+        if (requireNamespace ('rgdal', quietly = TRUE)) {
             xy <- as.matrix(xy)  ## required by rgdal, not proj4
             latlon <- rgdal::project (xy, inv = TRUE, proj)
         }
