@@ -51,8 +51,8 @@ List trappingsingle (
   double p;
   int nanimals;        // temporary 
   int ntraps;          // temporary 
-  int occupied[kk];    // today 
-  int intrap[N];       // today   
+  std::vector<int> occupied(kk);    // today 
+  std::vector<int> intrap(N);       // today   
   
   //struct  trap_animal *tran;
   std::vector<trap_animal> tran(N*kk);
@@ -422,7 +422,6 @@ List trappingcount (
   for (s=0; s<ss; s++) {
     for (i=0; i<N; i++) {
       for (k=0; k<kk; k++) {
-        // if (used[s * kk + k]) { 
         Tski = Tsk(k,s);
         if (fabs(Tski) > 1e-10) {          // nonzero 2012 12 18 
           gi = k * ss + s;
